package bcho.tracer.aspectj.aspects;



import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;

import bcho.tracer.logging.LoggingUnit;
import bcho.tracer.logging.Logger;
import bcho.tracer.logging.simplelogger.TextLogger;

public abstract aspect TracerAspect {
	pointcut targetClass() : within(bcho.sample..*) ;
	pointcut targetMethod() : targetClass() && execution(* *(..));
	
	ThreadMXBean threadMxBean = (ThreadMXBean) ManagementFactory.getThreadMXBean();
	Logger logger = new TextLogger();
	
	Object around(): targetMethod(){
		LoggingUnit unit = new LoggingUnit();
		String transactionId="empty";
		String localTransactionId = String.valueOf(System.nanoTime());
		if(Context.local.get() != null) transactionId=Context.local.get().toString();
		unit.setTransactionId(transactionId);
		unit.setLocalTransactionId(localTransactionId);
		unit.setStartTime(System.currentTimeMillis());
		unit.setStartCpuTime(threadMxBean.getCurrentThreadCpuTime());
		unit.setSignature(thisJoinPoint.getSignature());
		
		Context.push(unit);
		logger.logStart(unit);
		
		try{
			return proceed();
		}finally{
			unit.setEndTime(System.currentTimeMillis());
			unit.setEndCpuTime(threadMxBean.getCurrentThreadCpuTime());
				
			logger.logEnd(unit);
			Context.pop();

		}// try
	}
}
